/**
 * Wrap the runtime-specific hook into stdin
 */
import { ITerminalIO, ITerminalSize } from '../common/mod.ts';
import Ansi from '../common/editor/ansi.ts';

const BunTerminalIO: ITerminalIO = {
	inputLoop: async function* inputLoop() {
		for await (const chunk of Bun.stdin.stream()) {
			yield chunk;
		}
	},
	getTerminalSize: async function getTerminalSize(): Promise<ITerminalSize> {
		const encoder = new TextEncoder();
		const write = (s: string) => Bun.write(Bun.stdout, encoder.encode(s));

		// Tell the cursor to move to Row 999 and Column 999
		// Since this command specifically doesn't go off the screen
		// When we ask where the cursor is, we should get the size of the screen
		await write(Ansi.moveCursorForward(999) + Ansi.moveCursorDown(999));

		// Ask where the cursor is
		await write(Ansi.GetCursorLocation);

		// Get the first chunk from stdin
		// The response is \x1b[(rows);(cols)R..
		for await (const chunk of Bun.stdin.stream()) {
			const rawCode = (new TextDecoder()).decode(chunk);
			const res = rawCode.trim().replace(/^.\[([0-9]+;[0-9]+)R$/, '$1');
			const [srows, scols] = res.split(';');
			const rows = parseInt(srows, 10);
			const cols = parseInt(scols, 10);

			// Clear the screen
			await write(Ansi.ClearScreen + Ansi.ResetCursor);

			return {
				rows,
				cols,
			};
		}

		return {
			rows: 24,
			cols: 80,
		};
	},
	write: async function write(s: string): Promise<void> {
		const buffer = new TextEncoder().encode(s);

		await Bun.write(Bun.stdout, buffer);
	},
};

export default BunTerminalIO;
